#!/usr/bin/env python3
# Takes a list of JSON files containing annotations and extracts all unique labels
import click

@click.command()
@click.argument("input-files", nargs=-1, required=True, type=click.Path(exists=True, file_okay=True, dir_okay=False))
def main(input_files):
    from atlas.neuro.io import Annotations

    labels = []
    for fname in input_files:
        annotation = Annotations.from_file(fname)
        # Structures is a list of (name, array) tuples. We only need the names
        labels.extend([s[0] for s in annotation.structures])
    # Make labels unique
    labels = list(set(labels))
    # Sort them
    labels = sorted(labels)
    # Print them
    print(",".join(labels))

if __name__ == "__main__":
    main()