#!/bin/bash -x
#SBATCH --mail-user=c.schiffer@fz-juelich.de
#SBATCH --mail-type=FAIL
#SBATCH --nodes=1
#SBATCH --gres=gpu:4
#SBATCH --output=pipeline-job-out.%j
#SBATCH --error=pipeline-job-err.%j
#SBATCH --time=12:00:00
#SBATCH --partition=gpus

# Load modules
echo Loading modules...
module purge
module use ${SOFTWARE_STAGE_MODULES}
module load atlas
echo Modules loaded

# Set test set directory (assume we are in the experiment folder)
export ATLAS_TEST_SET_DIR=`pwd`/test_sets/
export ATLAS_ANNOTATION_DIR=/p/scratch/cjinm16/schiffer1/2019_06_03_labelpropagation_publication/annotations/

# Train network
srun -n 48 atlas dl train $@ --verbose --distribute

# Create predictions
srun -n 48 atlas dl predict $@ --verbose --output-spacing 16

# Perform evaluation
srun -n 24 atlas dl evaluate $@ --output-spacing 64 --write-overview --write-report --verbose --write-confusion-matrix --write-training-statistics
srun -n 24 atlas dl evaluate $@ --output-spacing 16 --embedded-pred-to-hdf5 --verbose
