if (debug) {
    console.log("atlas_ui.js loaded")
}

var atlasRegions = [];
var serverHost = "medpc018.ime.kfa-juelich.de"
var serverPort = 80
var serverEndpoint = "atlas_server/"
// var atlasId = null;

function Region(region, image) {
    this.name = region.name;
    this.region = region;
    this.image = image;
    this.id = `${currentImage}_${region.uid}`;
    // this.training = false;
    // this.submitted = false;
}

function compareRegions(region0, region1) {
    if (region0 === region1) {
        return true;
    }
    if (region0.id === region1.id) {
        return true;
    }
    return false;
}

function searchRegionInList(region, list) {
    for (var i in list) {
        if (compareRegions(list[i], region)) {
            return i;
        }
    }
    return -1;
}

function getTimestamp() {
    var date = new Date();
    return `${date.toLocaleDateString()} ${date.toLocaleTimeString()}`;
}

function post_message(msg) {
    var msg_box = $("#messages");
    var timestamp = getTimestamp();
    // msg_box.append(`<i>${timestamp}</i><br>${msg}<br>`);
    console.log(msg);
}

function removeRegionFromList(region, list) {
    let index = searchRegionInList(region, list);
    if (index != -1) {
        list.splice(index, 1);
    }
}

var app = new Vue({
    components: {
        ContextMenu,
    },
    el: '#atlas_app',
    data: {
        brain: 20,
        selected_regions: [],
        available_regions: [],
        atlas_id: null,
    },
    methods: {
        initializeAtlas: function () {
            this.checkOrGetAtlasId();
            this.refreshAvailableRegions();
        },
        checkOrGetAtlasId: function () {
            if (this.atlas_id === null) {
                this.getAtlasId();
            }
        },
        getAtlasId: function () {
            if (this.atlas_id !== null) {
                // We already have an id
                return;
            }
            var target = `http://${serverHost}:${serverPort}/${serverEndpoint}/request_id`;
            let _this = this;
            $.ajax({
                url: target,
                type: "POST",
                dataType: "json",
                crossDomain: true,
                contentType: "application/json",
                data: [],
                success: function (data) {
                    _this.atlas_id = data.id;
                    post_message(`ATLaS ID: ${_this.atlas_id}`);
                },
                error: function (jqXHR, textStatus, errorThrown) {
                    post_message(`Error getting id: ${textStatus} ${errorThrown}`);
                }
            });
        },
        refreshAvailableRegions: function () {
            // Clear available regions
            this.available_regions.splice(0, this.available_regions.length);
            // Add current regions
            var regions = ImageInfo[currentImage]["Regions"];
            var _this = this;
            for (var i in regions) {
                let region = new Region(regions[i], currentImage);
                if (searchRegionInList(region, this.selected_regions) != -1) {
                    // Is already selected
                    continue;
                }

                // Function to add this region to our selected regions list
                region.add = function () {
                    // Add to selected list
                    _this.selected_regions.push(region);
                    // Remove from available list
                    removeRegionFromList(region, _this.available_regions);

                };
                // Function to remove this region from our selected regions list
                region.remove = function () {
                    // Add to available list
                    _this.available_regions.push(region);
                    // Remove from selected list
                    removeRegionFromList(region, _this.selected_regions);
                };

                // Add region to the list of available regions
                this.available_regions.push(region);
            }
        },
        addRegion: function () {
            this.regions.push({
                "name": region.name,
                "region": region,
                "image": currentImage,
                "my_name": function () {
                    alert(region.name);
                },
            });
        },
        removeRegion: function () {
            var indices_to_remove = []
            for (var i in this.regions) {
                var region_in_list = this.regions[i];
                if (region_in_list.image == currentImage && region_in_list.region == region) {
                    indices_to_remove.push(i);
                }
            }
            // Reverse to not mess up indices when removing
            indices_to_remove.reverse();
            for (var i in indices_to_remove) {
                this.regions.splice(indices_to_remove[i], 1);
            }
        },
        setupTraining: function () {
            // Prepare annotations for transfer to server
            let annotations = [];
            for (var i in this.selected_regions) {
                let region = this.selected_regions[i];

                annotations.push({
                    "image": region.image,
                    "name": region.name,
                    "path": JSON.parse(region.region.path.exportJSON()),
                });
            }

            // Assemble JSON data to send
            data = {
                "id": this.atlas_id,
                "brain": this.brain,
                "annotations": annotations,
            }

            var payload = JSON.stringify(data);
            var target = `http://${serverHost}:${serverPort}/${serverEndpoint}/setup_training`;
            post_message(`Sending to ${target}`);
            $.ajax({
                url: target,
                type: "POST",
                dataType: "json",
                contentType: "application/json",
                data: payload,
                crossDomain: true,
                success: function (data) {
                    post_message(`Setup training requested!`);
                    alert(`Response: ${data}`)
                },
                error: function (jqXHR, textStatus, errorThrown) {
                    post_message(`Error: ${textStatus} ${errorThrown}`);
                }
            });
        },
    },
});